/*******************************************************************************
* Copyright (C) 2001-2025 Altera Corporation
*
* This code and the related documents are Altera copyrighted materials and
* your use of them is governed by the express license under which they were
* provided to you ("License"). Unless the License provides otherwise, you may
* not use, modify, copy, publish, distribute, disclose, or transmit this
* code or the related documents without Altera's prior written permission.
* 
* This code and the related documents are provided as is, with no express
* or implied warranties other than those that are expressly stated in the
* License.
* 
*******************************************************************************/


module bts_config (   
  input          refclk_gxep0,    // refclk for bank9A, to PRESERVE_UNUSED_XCVR_CHANNEL 
  input          fpga_system_clk, // system clock, 100M
  input          fpga_sys_resetn, //sysmax_pb[1] to trigger hard reset
  input [7:0]    fpga_sw,         //Need add internal WEAK_PULL_UP_RESISTOR
  input [7:0]    fpga_pb,
  output [7:0]   fpga_led_g,
  
  //QSFPDD control signals      
  input          qsfpdd_modprsn,
  input          qsfpdd_intn,
  output         qsfpdd_initmode,
  output         qsfpdd_modseln,
  output         qsfpdd_resetn,
  
  //QSFP control signals
  input          qsfp_modprsn,
  input          qsfp_intn,
  output         qsfp_initmode,
  output         qsfp_modseln,
  output         qsfp_resetn,
  
  inout          clkcleaner_scl,
  inout          clkcleaner_sda,
  inout          sodimm_i2c_sda,
  inout          sodimm_i2c_scl,
  inout          usb_scl,
  inout          usb_sda,
  inout          qsfpdd_i2c_scl,
  inout          qsfpdd_i2c_sda,
  inout          qsfp_i2c_scl,
  inout          qsfp_i2c_sda,
  inout          f2m_i2c_scl,
  inout          f2m_i2c_sda,
  inout          m2f_i2c_scl,
  inout          m2f_i2c_sda,
  inout          fpga_test_scl,
  inout          fpga_test_sda
  
);
  wire        ninit_done;	
  wire        S_global_resetn;	 
  reg  [26:0] heart_beat_cnt;
  wire        sys_pll_locked;
  wire        clk_50m;
  wire        clkcleaner_scl_in;
  wire        clkcleaner_sda_in;
  wire        clkcleaner_scl_oe;
  wire        clkcleaner_sda_oe;
  wire        sodimm_scl_in;
  wire        sodimm_sda_in;
  wire        sodimm_scl_oe;
  wire        sodimm_sda_oe;
  wire        usb_scl_in;
  wire        usb_sda_in;
  wire        usb_scl_oe;
  wire        usb_sda_oe;
  wire        qsfpp_scl_in;
  wire        qsfpp_sda_in;
  wire        qsfpp_scl_oe;
  wire        qsfpp_sda_oe;
  wire        qsfp_scl_in;
  wire        qsfp_sda_in;
  wire        qsfp_scl_oe;
  wire        qsfp_sda_oe;
  wire        f2m_scl_in;
  wire        f2m_sda_in;
  wire        f2m_scl_oe;
  wire        f2m_sda_oe;
  wire        m2f_scl_in;
  wire        m2f_sda_in;
  wire        m2f_scl_oe;
  wire        m2f_sda_oe;
  wire        fpga_test_scl_in;
  wire        fpga_test_sda_in;
  wire        fpga_test_scl_oe;
  wire        fpga_test_sda_oe;
  wire[7:0]   source;
  wire[3:0]   probe;
  
   sp sp(
	 .source    (source),  //initial 0001_0001
	 .probe     (probe)
	);
	
  assign  qsfpdd_resetn     = source[0]; //1'b1;  
  assign  qsfpdd_modseln    = source[1]; //1'b0;  
  assign  qsfpdd_initmode   = source[2]; //1'b0;  
  
  assign  qsfp_resetn       = source[4]; //1'b1;
  assign  qsfp_modseln      = source[5]; //1'b0;
  assign  qsfp_initmode     = source[6]; //1'b0;
  assign  probe[0] = qsfpdd_modprsn;
  assign  probe[1] = qsfpdd_intn;
  assign  probe[2] = qsfp_modprsn;
  assign  probe[3] = qsfp_intn;
  
  assign  clkcleaner_scl_in = clkcleaner_scl;
  assign  clkcleaner_sda_in = clkcleaner_sda;
  assign  clkcleaner_scl    = clkcleaner_scl_oe ? 1'b0 : 1'bz;
  assign  clkcleaner_sda    = clkcleaner_sda_oe ? 1'b0 : 1'bz;
  
  assign  sodimm_scl_in     = sodimm_i2c_scl;
  assign  sodimm_sda_in     = sodimm_i2c_sda;
  assign  sodimm_i2c_scl    = sodimm_scl_oe ? 1'b0 : 1'bz;
  assign  sodimm_i2c_sda    = sodimm_sda_oe ? 1'b0 : 1'bz;
  
  
  assign  qsfpdd_scl_in     = qsfpdd_i2c_scl;
  assign  qsfpdd_sda_in     = qsfpdd_i2c_sda;
  assign  qsfpdd_i2c_scl    = qsfpdd_scl_oe ? 1'b0 : 1'bz;
  assign  qsfpdd_i2c_sda    = qsfpdd_sda_oe ? 1'b0 : 1'bz;
  
  assign  qsfp_scl_in       = qsfp_i2c_scl;
  assign  qsfp_sda_in       = qsfp_i2c_sda;
  assign  qsfp_i2c_scl      = qsfp_scl_oe ? 1'b0 : 1'bz;
  assign  qsfp_i2c_sda      = qsfp_sda_oe ? 1'b0 : 1'bz;
 
//assign  usb_scl_in        = usb_scl;
//assign  usb_sda_in        = usb_sda;
//assign  usb_scl           = usb_scl_oe ? 1'b0 : 1'bz;
//assign  usb_sda           = usb_sda_oe ? 1'b0 : 1'bz;
//
//assign  f2m_scl_in       = f2m_i2c_scl;
//assign  f2m_sda_in       = f2m_i2c_sda;
//assign  f2m_i2c_scl      = f2m_scl_oe ? 1'b0 : 1'bz;
//assign  f2m_i2c_sda      = f2m_sda_oe ? 1'b0 : 1'bz;
//
//assign  m2f_scl_in       = m2f_i2c_scl;
//assign  m2f_sda_in       = m2f_i2c_sda;
//assign  m2f_i2c_scl      = m2f_scl_oe ? 1'b0 : 1'bz;
//assign  m2f_i2c_sda      = m2f_sda_oe ? 1'b0 : 1'bz;
//wire[7:0]  fpga_led_g_t;
//assign     fpga_led_g[0]   = sys_pll_locked;
//assign     fpga_led_g[1]   = heart_beat_cnt[26];
//assign     fpga_led_g[7:2] = fpga_led_g_t[7:2];
  
  
assign S_global_resetn = !ninit_done && fpga_sys_resetn;
reset_release reset_release (
	.ninit_done(ninit_done)
	);
 
  sys_pll sys_pll_inst(
  	.rst      (!S_global_resetn), 
  	.refclk   (fpga_system_clk ),
  	.locked   (sys_pll_locked  ),
  	.outclk_0 (clk_50m         )
  	); 
	
  always @(posedge clk_50m or negedge S_global_resetn)
      if (!S_global_resetn)
          heart_beat_cnt <= 27'h0; //0x7ffffff
      else
          heart_beat_cnt <= heart_beat_cnt + 1'b1;	
  		
  
  q_sys q_sys_inst(                                                               
      .clk_clk                                    (clk_50m           ),
      .reset_reset_n                              (sys_pll_locked    ), 
      .i2c_clkcleaner_i2c_serial_sda_in           (clkcleaner_sda_in ),
      .i2c_clkcleaner_i2c_serial_scl_in           (clkcleaner_scl_in ),
      .i2c_clkcleaner_i2c_serial_sda_oe           (clkcleaner_sda_oe ),
      .i2c_clkcleaner_i2c_serial_scl_oe           (clkcleaner_scl_oe ),
      .i2c_qsfp_i2c_serial_sda_in                 (qsfp_sda_in       ),
      .i2c_qsfp_i2c_serial_scl_in                 (qsfp_scl_in       ),
      .i2c_qsfp_i2c_serial_sda_oe                 (qsfp_sda_oe       ),
      .i2c_qsfp_i2c_serial_scl_oe                 (qsfp_scl_oe       ),
      .i2c_qsfpdd_i2c_serial_sda_in               (qsfpdd_sda_in     ),
      .i2c_qsfpdd_i2c_serial_scl_in               (qsfpdd_scl_in     ),
      .i2c_qsfpdd_i2c_serial_sda_oe               (qsfpdd_sda_oe     ),
      .i2c_qsfpdd_i2c_serial_scl_oe               (qsfpdd_scl_oe     ),
      .i2c_sodimm_i2c_serial_sda_in               (sodimm_sda_in     ),
      .i2c_sodimm_i2c_serial_scl_in               (sodimm_scl_in     ),
      .i2c_sodimm_i2c_serial_sda_oe               (sodimm_sda_oe     ),
      .i2c_sodimm_i2c_serial_scl_oe               (sodimm_scl_oe     ),
    //.i2c_f2m_i2c_serial_sda_in                  (f2m_sda_in        ),
    //.i2c_f2m_i2c_serial_scl_in                  (f2m_scl_in        ),
    //.i2c_f2m_i2c_serial_sda_oe                  (f2m_sda_oe        ),
    //.i2c_f2m_i2c_serial_scl_oe                  (f2m_scl_oe        ),
    //.i2c_m2f_i2c_serial_sda_in                  (m2f_sda_in        ),
    //.i2c_m2f_i2c_serial_scl_in                  (m2f_scl_in        ),
    //.i2c_m2f_i2c_serial_sda_oe                  (m2f_sda_oe        ),
    //.i2c_m2f_i2c_serial_scl_oe                  (m2f_scl_oe        ),
    //.i2c_usb_i2c_serial_sda_in                  (usb_sda_in        ),
    //.i2c_usb_i2c_serial_scl_in                  (usb_scl_in        ),
    //.i2c_usb_i2c_serial_sda_oe                  (usb_sda_oe        ),
    //.i2c_usb_i2c_serial_scl_oe                  (usb_scl_oe        ),
      .pio_0_user_led_external_connection_export  (fpga_led_g        ),//fpga_led_g_t      ),
      .pio_1_user_dipsw_external_connection_export(fpga_sw           ),
      .pio_2_user_pb_external_connection_export   (fpga_pb           ) 
  	);		                                                                    
		
endmodule	
	