/*******************************************************************************
* Copyright (C) 2001-2025 Altera Corporation
*
* This code and the related documents are Altera copyrighted materials and
* your use of them is governed by the express license under which they were
* provided to you ("License"). Unless the License provides otherwise, you may
* not use, modify, copy, publish, distribute, disclose, or transmit this
* code or the related documents without Altera's prior written permission.
* 
* This code and the related documents are provided as is, with no express
* or implied warranties other than those that are expressly stated in the
* License.
* 
*******************************************************************************/


module sodim (   
  input  wire         FPGA_SYS_RESETN,    // sysmax_pb[1] to trigger hard reset
  input  wire         FPGA_SYSTEM_CLK,    // system clock, 100M
  input  wire         REFCLK_GXEP0,       // refclk for bank9A, to PRESERVE_UNUSED_XCVR_CHANNEL   
  
  input  wire         SODIMM_RZQ,
  input  wire         SODIMM_REFCLKP,
  output wire         MEM_CLKN,
  output wire         MEM_CLKP,
  inout  wire [31:0]  MEM_DQA,  // 33,32 are reserved for CS3,2, useless
  inout  wire [31:0]  MEM_DQB,  // 33,32 are reserved for CK1, useless
  inout  wire [3:0]   MEM_DQSAN,
  inout  wire [3:0]   MEM_DQSAP,
  inout  wire [3:0]   MEM_DQSBN,
  inout  wire [3:0]   MEM_DQSBP,
  inout  wire [3:0]   MEM_DMAN,
  inout  wire [3:0]   MEM_DMBN,
  inout  wire         MEM_DQS_ADDR_CMDN,
  inout  wire         MEM_DQS_ADDR_CMDP,
  inout  wire [8:0]   MEM_DQ_ADDR_CMD,
  output wire [26:0]  MEM_ADDR_CMD,   
  input  wire         MEM_ADDR_CMD27,
  output wire         MEM_ADDR_CMD28,
  output wire         MEM_ADDR_CMD29,
  output wire         MEM_ADDR_CMD30,
  output wire [7:0]   FPGA_LED_G
);
  wire   ninit_done;
  wire   S_global_resetn;
  wire   cal_success;         
  wire   cal_fail;           
  wire   local_reset_done;    
  wire   error_mon;    
  wire   status_mon;     
  wire   sys_pll_locked;
  wire   clk_50m;
  reg  [26:0] heart_beat_cnt;
  
  assign MEM_ADDR_CMD[5] = 1'bZ;
  assign MEM_ADDR_CMD[7] = 1'bZ;
  assign MEM_ADDR_CMD[8] = 1'bZ;
  
//led, low active
  assign FPGA_LED_G[0] = heart_beat_cnt[26];     //green  Blinking  
  assign FPGA_LED_G[1] = status_mon;             //green  OFF
  assign FPGA_LED_G[2] = !sys_pll_locked;        //green  ON
  assign FPGA_LED_G[3] = S_global_resetn;        //green  OFF  
  assign FPGA_LED_G[4] = !error_mon;             //red    OFF     
  assign FPGA_LED_G[5] = 1'b1;                   //red    OFF
  assign FPGA_LED_G[6] = !ninit_done;            //amber  OFF
  assign FPGA_LED_G[7] = local_reset_done;       //amber  OFF 
  assign S_global_resetn = !ninit_done && FPGA_SYS_RESETN;
  reset_release reset_release (
  	.ninit_done(ninit_done)
  	);
  	
  sys_pll sys_pll_inst(
  	.rst      (!S_global_resetn), 
  	.refclk   (FPGA_SYSTEM_CLK ),
  	.locked   (sys_pll_locked  ),
  	.outclk_0 (clk_50m         )
  	);
  
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Heart-beat indicate
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
always @(posedge clk_50m or negedge sys_pll_locked)
    if (!sys_pll_locked)
        heart_beat_cnt <= 27'h0; //0x7FFFFFF
    else
        heart_beat_cnt <= heart_beat_cnt + 1'b1;
ed_synth ed_synth_inst(     
	.clk_50m_clk                             (clk_50m                                ), 
	.global_reset_reset_n                    (sys_pll_locked                         ),
	.emif_fm_0_oct_oct_rzqin                 (SODIMM_RZQ                             ),
	.emif_fm_0_mem_mem_ck                    (MEM_CLKP                               ),
	.emif_fm_0_mem_mem_ck_n                  (MEM_CLKN                               ),
	.emif_fm_0_mem_mem_a                     (MEM_ADDR_CMD[26:10]                    ),
	.emif_fm_0_mem_mem_act_n                 (MEM_ADDR_CMD[3]                        ),
	.emif_fm_0_mem_mem_ba                    ({MEM_ADDR_CMD29,MEM_ADDR_CMD28}        ),
	.emif_fm_0_mem_mem_bg                    ({MEM_ADDR_CMD[0],MEM_ADDR_CMD30}       ),
	.emif_fm_0_mem_mem_cke                   (MEM_ADDR_CMD[6]                        ),
	.emif_fm_0_mem_mem_cs_n                  (MEM_ADDR_CMD[2]                        ),
	.emif_fm_0_mem_mem_odt                   (MEM_ADDR_CMD[4]                        ),
	.emif_fm_0_mem_mem_reset_n               (MEM_ADDR_CMD[1]                        ),
	.emif_fm_0_mem_mem_par                   (MEM_ADDR_CMD[9]                        ),
	.emif_fm_0_mem_mem_alert_n               (MEM_ADDR_CMD27                         ),
	.emif_fm_0_mem_mem_dqs                   ({MEM_DQS_ADDR_CMDP,MEM_DQSBP,MEM_DQSAP}),
	.emif_fm_0_mem_mem_dqs_n                 ({MEM_DQS_ADDR_CMDN,MEM_DQSBN,MEM_DQSAN}),
	.emif_fm_0_mem_mem_dq                    ({MEM_DQ_ADDR_CMD[0],MEM_DQ_ADDR_CMD[3],MEM_DQ_ADDR_CMD[8],MEM_DQ_ADDR_CMD[5],MEM_DQ_ADDR_CMD[1],MEM_DQ_ADDR_CMD[2],MEM_DQ_ADDR_CMD[6],MEM_DQ_ADDR_CMD[7],MEM_DQB[31:0],MEM_DQA[31:0]}),
	.emif_fm_0_mem_mem_dbi_n                 ({MEM_DQ_ADDR_CMD[4],MEM_DMBN,MEM_DMAN} ),
	.emif_fm_0_pll_ref_clk_clk               (SODIMM_REFCLKP                         ), 
	.local_reset_req                         (sys_pll_locked                         ),
	.local_reset_done                        (local_reset_done                       ),
	.master_driver_msgdma_0_error_mon_export (error_mon                              ),
	.master_driver_msgdma_0_status_mon_export(status_mon                             )
	);
		 
endmodule	
	
