/*******************************************************************************
* Copyright (C) 2001-2025 Altera Corporation
*
* This code and the related documents are Altera copyrighted materials and
* your use of them is governed by the express license under which they were
* provided to you ("License"). Unless the License provides otherwise, you may
* not use, modify, copy, publish, distribute, disclose, or transmit this
* code or the related documents without Altera's prior written permission.
* 
* This code and the related documents are provided as is, with no express
* or implied warranties other than those that are expressly stated in the
* License.
* 
*******************************************************************************/


`timescale 1ns / 100ps
module  xcvr (
        //System clock and reset
        input             FPGA_SYSTEM_CLK,    // system clock, 100M
        input             FPGA_GPIO_REFCLK,   // Refclk 100M
        input             REFCLK_GXEP0,       // Refclk_9A, 156.35M
        input             FPGA_SYS_RESETN,    // sysmax_pb[1] to trigger hard reset
        output  [7:0]     FPGA_LED_G,        
		//MXP		
        input  [3:0]      MXP_RXP,         
        input  [3:0]      MXP_RXN,         
        output [3:0]      MXP_TXP,         
        output [3:0]      MXP_TXN, 
		
		//ZQSFP		
        input  [3:0]      ZQSFP_RXP,         
        input  [3:0]      ZQSFP_RXN,         
        output [3:0]      ZQSFP_TXP,         
        output [3:0]      ZQSFP_TXN, 
		
        //QSFPDD
        input  [7:0]      QSFPDD_RXP,         
        input  [7:0]      QSFPDD_RXN,         
        output [7:0]      QSFPDD_TXP,         
        output [7:0]      QSFPDD_TXN,         
        input             QSFPDD_MODPRSN,
        input             QSFPDD_INTN,
        output            QSFPDD_INITMODE,
        output            QSFPDD_MODSELN,
        output            QSFPDD_RESETN,
        inout             QSFPDD_I2C_SCL,
        inout             QSFPDD_I2C_SDA 
        );
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Signal definitions
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reg  [26:0] heart_beat_cnt;
wire [7:0]  module_debug_outputs;
wire [7:0]  module_debug_inputs;
wire [31:0] fpga_core_temp;
wire [31:0] fpga_xcvr_tile0_temp;
wire [31:0] fpga_xcvr_tile1_temp;
wire [31:0] fpga_xcvr_tile2_temp;
wire [31:0] fpga_xcvr_tile3_temp;
wire [31:0] fpga_xcvr_tile4_temp;
wire [31:0] fpga_xcvr_tile5_temp;
wire [31:0] fpga_max_temp;
wire        qsfpdd_scl_in;
wire        qsfpdd_sda_in;
wire        qsfpdd_scl_oe;
wire        qsfpdd_sda_oe;
wire        iopll_0_locked;
wire        iopll_1_locked;
wire        iopll_2_locked;
wire        iopll_3_locked;
wire        sys_pll_locked/* synthesis keep="1" */;
wire        clk_50m;
wire        ninit_done;
wire        S_global_resetn/* synthesis keep="1" */;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Input/output assignments
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  reset_release reset_release (
  	.ninit_done(ninit_done)
  	);
  	
assign  S_global_resetn = !ninit_done && FPGA_SYS_RESETN; 
assign  qsfpdd_scl_in     = QSFPDD_I2C_SCL;
assign  qsfpdd_sda_in     = QSFPDD_I2C_SDA;
assign  QSFPDD_I2C_SCL    = qsfpdd_scl_oe ? 1'b0 : 1'bz;
assign  QSFPDD_I2C_SDA    = qsfpdd_sda_oe ? 1'b0 : 1'bz;
assign  QSFPDD_INITMODE   = module_debug_outputs[0];
assign  QSFPDD_MODSELN    = module_debug_outputs[1];
assign  QSFPDD_RESETN     = module_debug_outputs[2];
assign  module_debug_inputs[0]   = QSFPDD_MODPRSN;
assign  module_debug_inputs[1]   = QSFPDD_INTN;
assign  module_debug_inputs[7:2] = 6'd0;
   
//LED[3:0] - Green; LED[5:4] - Red; LED[7:6]- Amber, low active   
assign  FPGA_LED_G[0]   = heart_beat_cnt[26];                  //green  Blinking   
assign  FPGA_LED_G[1]   = !sys_pll_locked ;                    //green  ON
assign  FPGA_LED_G[2]   = !(iopll_0_locked && iopll_1_locked); //green  ON
assign  FPGA_LED_G[3]   = !(iopll_2_locked && iopll_3_locked); //green  ON 
assign  FPGA_LED_G[4]   = 1'b1;                                //red    OFF     
assign  FPGA_LED_G[5]   = FPGA_SYS_RESETN;                     //red    OFF
assign  FPGA_LED_G[6]   = !ninit_done;                         //amber  OFF
assign  FPGA_LED_G[7]   = 1'b1;                                //amber  OFF 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Heart-beat indicate
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
always @(posedge clk_50m or negedge sys_pll_locked)
    if (!sys_pll_locked)
        heart_beat_cnt <= 27'h0; //0x7FFFFFF
    else
        heart_beat_cnt <= heart_beat_cnt + 1'b1;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//System pll
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
sys_pll sys_pll_inst(
	.rst      (!S_global_resetn), 
	.refclk   (FPGA_SYSTEM_CLK ),
	.locked   (sys_pll_locked  ),
	.outclk_0 (clk_50m         )
	);
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//FPGA on-chip TSD interface
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//fpga_tsd_monitor    fpga_tsd_monitor_inst0
//    (
//    .i_clk                      (clk_50m             ), // Clock for i2c slave interface
//    .i_clk_rstn                 (sys_pll_locked      ), // Reset, high active
//    .o_fpga_core_temp           (fpga_core_temp      ), // FPGA core temperature
//    .o_fpga_xcvr_tile0_temp     (fpga_xcvr_tile0_temp), // FPGA XCVR Tile #0 temperature if any
//    .o_fpga_xcvr_tile1_temp     (fpga_xcvr_tile1_temp), // FPGA XCVR Tile #1 temperature if any
//    .o_fpga_xcvr_tile2_temp     (fpga_xcvr_tile2_temp), // FPGA XCVR Tile #2 temperature if any
//    .o_fpga_xcvr_tile3_temp     (fpga_xcvr_tile3_temp), // FPGA XCVR Tile #3 temperature if any
//    .o_fpga_xcvr_tile4_temp     (fpga_xcvr_tile4_temp), // FPGA XCVR Tile #4 temperature if any
//    .o_fpga_xcvr_tile5_temp     (fpga_xcvr_tile5_temp), // FPGA XCVR Tile #5 temperature if any
//    .o_fpga_max_temp            (fpga_max_temp       )  // FPGA maximum temperature
//    /*
//    .o_user_ready               (                    ), // user_ready
//    .o_rsp_startofpacket        (                    ), // rsp_startofpacket
//    .o_rsp_endofpacket          (                    )  // rsp_endofpacket
//    */
//    );
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//XCVR system Qsys module
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
q_sys   q_sys_inst0        (
    .clk_100_clk                                                         (FPGA_GPIO_REFCLK    ),       
    .reset_100_reset_n                                                   (S_global_resetn     ),
    .clk_50_clk                                                          (clk_50m             ),
    .reset_50_reset_n                                                    (sys_pll_locked      ),
    .i2c_0_i2c_serial_sda_in                                             (qsfpdd_sda_in       ),
    .i2c_0_i2c_serial_scl_in                                             (qsfpdd_scl_in       ),
    .i2c_0_i2c_serial_sda_oe                                             (qsfpdd_sda_oe       ),
    .i2c_0_i2c_serial_scl_oe                                             (qsfpdd_scl_oe       ),
    .iopll_0_locked_export                                               (iopll_0_locked      ),
    .iopll_1_locked_export                                               (iopll_1_locked      ),
    .iopll_2_locked_export                                               (iopll_2_locked      ),
    .iopll_3_locked_export                                               (iopll_3_locked      ),
    .module_debug_inputs_external_connection_export                      (module_debug_inputs ),
    .module_debug_outputs_external_connection_export                     (module_debug_outputs),
    .refclk_etile_9a_in_clk_clk                                          (REFCLK_GXEP0        ),
    .temp_info_0_fpga_core_temp_export                                   (fpga_core_temp      ),
    .temp_info_0_fpga_max_temp_export                                    (fpga_max_temp       ),
    .temp_info_0_fpga_xcvr_tile0_temp_export                             (fpga_xcvr_tile0_temp),
    .temp_info_0_fpga_xcvr_tile1_temp_export                             (fpga_xcvr_tile1_temp),
    .temp_info_0_fpga_xcvr_tile2_temp_export                             (fpga_xcvr_tile2_temp),
    .temp_info_0_fpga_xcvr_tile3_temp_export                             (fpga_xcvr_tile3_temp),
    .temp_info_0_fpga_xcvr_tile4_temp_export                             (fpga_xcvr_tile4_temp),
    .temp_info_0_fpga_xcvr_tile5_temp_export                             (fpga_xcvr_tile5_temp),    
    .mxp_xcvr_test_0_xcvrnphy_fme_0_reset_reset                          (!sys_pll_locked     ),
    .mxp_xcvr_test_0_xcvrnphy_fme_0_tx_serial_data_tx_serial_data        (MXP_TXP[0]          ),
    .mxp_xcvr_test_0_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n    (MXP_TXN[0]          ),
    .mxp_xcvr_test_0_xcvrnphy_fme_0_rx_serial_data_rx_serial_data        (MXP_RXP[0]          ),
    .mxp_xcvr_test_0_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n    (MXP_RXN[0]          ),
    .mxp_xcvr_test_1_xcvrnphy_fme_0_reset_reset                          (!sys_pll_locked     ),
    .mxp_xcvr_test_1_xcvrnphy_fme_0_tx_serial_data_tx_serial_data        (MXP_TXP[1]          ),
    .mxp_xcvr_test_1_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n    (MXP_TXN[1]          ),
    .mxp_xcvr_test_1_xcvrnphy_fme_0_rx_serial_data_rx_serial_data        (MXP_RXP[1]          ),
    .mxp_xcvr_test_1_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n    (MXP_RXN[1]          ),
    .mxp_xcvr_test_2_xcvrnphy_fme_0_reset_reset                          (!sys_pll_locked     ),
    .mxp_xcvr_test_2_xcvrnphy_fme_0_tx_serial_data_tx_serial_data        (MXP_TXP[2]          ),
    .mxp_xcvr_test_2_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n    (MXP_TXN[2]          ),
    .mxp_xcvr_test_2_xcvrnphy_fme_0_rx_serial_data_rx_serial_data        (MXP_RXP[2]          ),
    .mxp_xcvr_test_2_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n    (MXP_RXN[2]          ),
    .mxp_xcvr_test_3_xcvrnphy_fme_0_reset_reset                          (!sys_pll_locked     ),
    .mxp_xcvr_test_3_xcvrnphy_fme_0_tx_serial_data_tx_serial_data        (MXP_TXP[3]          ),
    .mxp_xcvr_test_3_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n    (MXP_TXN[3]          ),
    .mxp_xcvr_test_3_xcvrnphy_fme_0_rx_serial_data_rx_serial_data        (MXP_RXP[3]          ),
    .mxp_xcvr_test_3_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n    (MXP_RXN[3]          ),
    .qsfpdd_xcvr_test_0_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_0_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[0]       ),
    .qsfpdd_xcvr_test_0_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[0]       ),
    .qsfpdd_xcvr_test_0_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[0]       ),
    .qsfpdd_xcvr_test_0_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[0]       ),
    .qsfpdd_xcvr_test_1_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_1_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[1]       ),
    .qsfpdd_xcvr_test_1_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[1]       ),
    .qsfpdd_xcvr_test_1_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[1]       ),
    .qsfpdd_xcvr_test_1_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[1]       ),
    .qsfpdd_xcvr_test_2_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_2_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[2]       ),
    .qsfpdd_xcvr_test_2_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[2]       ),
    .qsfpdd_xcvr_test_2_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[2]       ),
    .qsfpdd_xcvr_test_2_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[2]       ),
    .qsfpdd_xcvr_test_3_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_3_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[3]       ),
    .qsfpdd_xcvr_test_3_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[3]       ),
    .qsfpdd_xcvr_test_3_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[3]       ),
    .qsfpdd_xcvr_test_3_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[3]       ),
    .qsfpdd_xcvr_test_4_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_4_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[4]       ),
    .qsfpdd_xcvr_test_4_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[4]       ),
    .qsfpdd_xcvr_test_4_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[4]       ),
    .qsfpdd_xcvr_test_4_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[4]       ),
    .qsfpdd_xcvr_test_5_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_5_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[5]       ),
    .qsfpdd_xcvr_test_5_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[5]       ),
    .qsfpdd_xcvr_test_5_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[5]       ),
    .qsfpdd_xcvr_test_5_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[5]       ),
    .qsfpdd_xcvr_test_6_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_6_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[6]       ),
    .qsfpdd_xcvr_test_6_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[6]       ),
    .qsfpdd_xcvr_test_6_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[6]       ),
    .qsfpdd_xcvr_test_6_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[6]       ),
    .qsfpdd_xcvr_test_7_xcvrnphy_fme_0_reset_reset                       (!sys_pll_locked     ),
    .qsfpdd_xcvr_test_7_xcvrnphy_fme_0_tx_serial_data_tx_serial_data     (QSFPDD_TXP[7]       ),
    .qsfpdd_xcvr_test_7_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n (QSFPDD_TXN[7]       ),
    .qsfpdd_xcvr_test_7_xcvrnphy_fme_0_rx_serial_data_rx_serial_data     (QSFPDD_RXP[7]       ),
    .qsfpdd_xcvr_test_7_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n (QSFPDD_RXN[7]       ),
    .zqsfp_xcvr_test_0_xcvrnphy_fme_0_reset_reset                        (!sys_pll_locked     ),
    .zqsfp_xcvr_test_0_xcvrnphy_fme_0_tx_serial_data_tx_serial_data      (ZQSFP_TXP[0]        ),
    .zqsfp_xcvr_test_0_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n  (ZQSFP_TXN[0]        ),
    .zqsfp_xcvr_test_0_xcvrnphy_fme_0_rx_serial_data_rx_serial_data      (ZQSFP_RXP[0]        ),
    .zqsfp_xcvr_test_0_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n  (ZQSFP_RXN[0]        ),
    .zqsfp_xcvr_test_1_xcvrnphy_fme_0_reset_reset                        (!sys_pll_locked     ),
    .zqsfp_xcvr_test_1_xcvrnphy_fme_0_tx_serial_data_tx_serial_data      (ZQSFP_TXP[1]        ),
    .zqsfp_xcvr_test_1_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n  (ZQSFP_TXN[1]        ),
    .zqsfp_xcvr_test_1_xcvrnphy_fme_0_rx_serial_data_rx_serial_data      (ZQSFP_RXP[1]        ),
    .zqsfp_xcvr_test_1_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n  (ZQSFP_RXN[1]        ),
    .zqsfp_xcvr_test_2_xcvrnphy_fme_0_reset_reset                        (!sys_pll_locked     ),
    .zqsfp_xcvr_test_2_xcvrnphy_fme_0_tx_serial_data_tx_serial_data      (ZQSFP_TXP[2]        ),
    .zqsfp_xcvr_test_2_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n  (ZQSFP_TXN[2]        ),
    .zqsfp_xcvr_test_2_xcvrnphy_fme_0_rx_serial_data_rx_serial_data      (ZQSFP_RXP[2]        ),
    .zqsfp_xcvr_test_2_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n  (ZQSFP_RXN[2]        ),
    .zqsfp_xcvr_test_3_xcvrnphy_fme_0_reset_reset                        (!sys_pll_locked     ),
    .zqsfp_xcvr_test_3_xcvrnphy_fme_0_tx_serial_data_tx_serial_data      (ZQSFP_TXP[3]        ),
    .zqsfp_xcvr_test_3_xcvrnphy_fme_0_tx_serial_data_n_tx_serial_data_n  (ZQSFP_TXN[3]        ),
    .zqsfp_xcvr_test_3_xcvrnphy_fme_0_rx_serial_data_rx_serial_data      (ZQSFP_RXP[3]        ),
    .zqsfp_xcvr_test_3_xcvrnphy_fme_0_rx_serial_data_n_rx_serial_data_n  (ZQSFP_RXN[3]        )
);
endmodule
	