// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


/*
xcvr_conduit_ctrl.v
Description:  
This component is used to contrl conduits of TX Native PHY IP Core
*/
module  xcvr_conduit_ctrl
        (
        input  wire                 reset_n,
        input  wire                 clk,
        input  wire [3:0]           csr_address,
        input  wire                 csr_read,
        input  wire                 csr_write,
        output reg  [31:0]          csr_readdata,
        input  wire [31:0]          csr_writedata,
        output wire                 rsfec_avmm2_avmmread_in,
        output wire                 rsfec_avmm2_avmmrequest_in,
        output wire                 rsfec_avmm2_avmmwrite_in,
        output wire                 latency_sclk,
        input  wire                 tx_dl_async_pulse,
        output wire                 tx_dl_measure_sel,
        input  wire                 tx_pma_ready,
        input  wire                 tx_ready,
        input  wire                 rx_dl_async_pulse,
        output wire                 rx_dl_measure_sel,
        input  wire                 rx_pma_ready,
        input  wire                 rx_ready
        );
    parameter TX_NATIVE_PHY_RSFEC_AVMM2_AVMMREAD_IN    = 0;
    parameter TX_NATIVE_PHY_RSFEC_AVMM2_AVMMREQUEST_IN = 0;
    parameter TX_NATIVE_PHY_RSFEC_AVMM2_AVMMWRITE_IN   = 0;
    parameter TX_NATIVE_PHY_LATENCY_SCLK               = 0;
    parameter TX_NATIVE_PHY_TX_DL_MEASURE_SEL          = 0;
    parameter TX_NATIVE_PHY_RX_DL_MEASURE_SEL          = 0;
    reg  [31:0] contrl;
    wire [31:0] tx_native_phy_settings;
    wire [31:0] tx_rx_status;
    assign rsfec_avmm2_avmmread_in    = TX_NATIVE_PHY_RSFEC_AVMM2_AVMMREAD_IN;
    assign rsfec_avmm2_avmmrequest_in = TX_NATIVE_PHY_RSFEC_AVMM2_AVMMREQUEST_IN;
    assign rsfec_avmm2_avmmwrite_in   = TX_NATIVE_PHY_RSFEC_AVMM2_AVMMWRITE_IN;
    assign latency_sclk               = TX_NATIVE_PHY_LATENCY_SCLK;
    assign tx_dl_measure_sel          = TX_NATIVE_PHY_TX_DL_MEASURE_SEL;
    assign rx_dl_measure_sel          = TX_NATIVE_PHY_RX_DL_MEASURE_SEL;
    assign tx_native_phy_settings[0]    = rsfec_avmm2_avmmread_in;
    assign tx_native_phy_settings[1]    = rsfec_avmm2_avmmrequest_in;
    assign tx_native_phy_settings[2]    = rsfec_avmm2_avmmwrite_in;
    assign tx_native_phy_settings[3]    = latency_sclk;
    assign tx_native_phy_settings[4]    = tx_dl_measure_sel;
    assign tx_native_phy_settings[5]    = rx_dl_measure_sel;
    assign tx_native_phy_settings[31:6] = 26'h0000000;
    assign tx_rx_status[0]    = tx_dl_async_pulse;
    assign tx_rx_status[1]    = tx_pma_ready;
    assign tx_rx_status[2]    = tx_ready;
    assign tx_rx_status[3]    = 1'b0;
    assign tx_rx_status[4]    = rx_dl_async_pulse;
    assign tx_rx_status[5]    = rx_pma_ready;
    assign tx_rx_status[6]    = rx_ready;
    assign tx_rx_status[7]    = 1'b0;
    assign tx_rx_status[31:8] = 24'h000000;
	
    always @ (posedge clk or negedge reset_n)begin
        if (reset_n == 0)begin
            contrl <= 32'h00000000;
        end else if(csr_write == 1)begin
            case(csr_address)
                0:contrl <= csr_writedata;
                default:;
            endcase
        end
    end
    always @ (posedge clk or negedge reset_n)begin
        if (reset_n == 0)begin
            csr_readdata <= 0;
        end else if(csr_read == 1)begin
            case(csr_address)
                0:csr_readdata <= contrl;
                1:csr_readdata <= tx_native_phy_settings;
                2:csr_readdata <= tx_rx_status;
                default:csr_readdata <= 0;
            endcase
        end
    end
endmodule
