// INTEL CONFIDENTIAL
// Copyright 2019 Intel Corporation.  All Rights Reserved.
//
// The source code contained or described herein and all documents related
// to the source code ("Material") are owned by Intel Corporation or its
// suppliers or licensors. Title to the Material remains with Intel
// Corporation or its suppliers and licensors. The Material contains trade
// secrets and proprietary and confidential information of Intel or its
// suppliers and licensors. The Material is protected by worldwide copyright
// and trade secret laws and treaty provisions. No part of the Material may
// be used, copied, reproduced, modified, published, uploaded, posted,
// transmitted, distributed, or disclosed in any way without Intel's prior
// express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or
// delivery of the Materials, either expressly, by implication, inducement,
// estoppel or otherwise. Any license under such intellectual property rights
// must be express and approved by Intel in writing.

`timescale 1ns / 100ps

//PWM Frequency 25KHz, actually 24.4KHz here for easier control

module  fan_controller
        (
        input  wire        i_clk          , // reset
        input  wire        i_reset_n      , // clock
        input  wire [2:0]  i_pwm_duty_ctrl, // Duty cycle control, 3'b100 (60%) by default
        output wire        o_fan_cont_out   // Fan control output, low active
        );

//--------------------------------------------------
// Signal defination
//--------------------------------------------------
wire        clk;
wire        reset_n;
wire        clk_pls_p;
wire [2:0]  pwm_duty_ctrl;
reg         fan_cont_out;

reg  [10:0] pwm_counter;  // 0~2047
reg  [10:0] pwm_duty_cnt; // 0~2047

assign  clk            = i_clk;
assign  reset_n        = i_reset_n;
assign  pwm_duty_ctrl  = i_pwm_duty_ctrl;
assign  o_fan_cont_out = fan_cont_out;

//--------------------------------------------------
// Main function body
//--------------------------------------------------
always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
     pwm_counter <= 11'd0;
  else
     pwm_counter <= pwm_counter + 1'b1;
end

always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
     pwm_duty_cnt <= 11'd0;
  else
    begin
      case (pwm_duty_ctrl)
      3'b000:  pwm_duty_cnt <= 11'd408;  // 20%
      3'b001:  pwm_duty_cnt <= 11'd613;  // 30%
      3'b010:  pwm_duty_cnt <= 11'd818;  // 40%
      3'b011:  pwm_duty_cnt <= 11'd1023; // 50%
      3'b100:  pwm_duty_cnt <= 11'd1228; // 60%
      3'b101:  pwm_duty_cnt <= 11'd1432; // 70%
      3'b110:  pwm_duty_cnt <= 11'd1637; // 80%
      3'b111:  pwm_duty_cnt <= 11'd2047; // 100%
      default: pwm_duty_cnt <= 11'd0;    // 0%
      endcase
    end
end

always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
     fan_cont_out <= 1'b1;
  else if(pwm_counter > pwm_duty_cnt)
     fan_cont_out <= 1'b1;
  else
     fan_cont_out <= 1'b0;
end

endmodule
//------------------------------------------------------------------------------
// end of file
//------------------------------------------------------------------------------