// INTEL CONFIDENTIAL
// Copyright 2019 Intel Corporation.  All Rights Reserved.
//
// The source code contained or described herein and all documents related
// to the source code ("Material") are owned by Intel Corporation or its
// suppliers or licensors. Title to the Material remains with Intel
// Corporation or its suppliers and licensors. The Material contains trade
// secrets and proprietary and confidential information of Intel or its
// suppliers and licensors. The Material is protected by worldwide copyright
// and trade secret laws and treaty provisions. No part of the Material may
// be used, copied, reproduced, modified, published, uploaded, posted,
// transmitted, distributed, or disclosed in any way without Intel's prior
// express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or
// delivery of the Materials, either expressly, by implication, inducement,
// estoppel or otherwise. Any license under such intellectual property rights
// must be express and approved by Intel in writing.

module max10_system( 
  inout        sdm_io0              , //pwr_scl   ?  reserved
  input        hps_gpio28_r         , //B_5, hps_io_gpio1_io4 ?    reserved
  input        hps_gpio29_r         , //B_6, hps_io_gpio1_io5 ?    reserved
  inout        hps_gpio30           , //B_7, hps_io_I2C1_SDA  ?    reserved
  inout        hps_gpio31           , //B_8, hps_io_I2C1_SCL  ?    reserved
  output       hps_dc_resetn        ,
  input        hps_proc_rstn        , //monitor, warm reset
  input        hps_dc_prsntn        ,
  
  input        pg_sysmax            , //SYSMAX power good, Include3.3V and 1.8V. 
  input        pg_fpga              , //FPGA power good 
  input        clk_50m_sysmax       , //50MHz global clock 
  output       fpga_sys_resetn      , //FPGA SYSTEM RESET
  
 //SW, PB & LEDs
  input [5:0]  sysmax_sw            , //DipSW
  output[4:0]  sysmax_led_g         , //Green Leds
  output[1:0]  sysmax_led_y         , //Yellow Leds
  input [5:0]  sysmax_pb            , //Push buttons
  input[15:0]  fpga_gpio            , //GPIOs between system MAX10 and FM
 
//Flash+configuration
  input        fpga_msel_en         , //MSEL SW control enable,reserved
  input [2:0]  fpga_msel            , //MSEL invert bits  
  input        sdm_io16             , //CONFIG_DONE   
  input        avst_ready           ,
  output       avst_data_valid      ,
  output       avst_clk             ,
  output[31:0] avst_d               ,
  output       fpga_nconfig         ,
  input        fpga_nstatus         ,
  
  
//used as AS configuration flash by default, may be used together with DC1 for CFI flash 
  output       flash_dc0_rstn       ,
  input        flash_dc0_prsntn     ,
  inout[9:0]   flash_dc0_io         ,
  inout        sdm_io15             ,//flash_dc0_io10 from sdm_io15, used as flash_dc0_rstn
  inout        flash_dc0_io11       ,
  inout        hps_crstn            ,//flash_dc0_io12 to sdm_io11, hps cold reset
  inout[39:13] flash_dc0_io39_13    ,

 //AVST  - QSPI                                   
  output       flash_dc1_rstn       , 
  input        flash_dc1_prsntn     ,
//inout[39:0]  flash_dc1_io         ,  
  inout        flash_dc1_io1        , //QSPI_IO0
  inout        flash_dc1_io3        , //QSPI_IO1
  inout        flash_dc1_io4        , //QSPI_IO2
  inout        flash_dc1_io5        , //QSPI_IO3
  output       flash_dc1_io2        , //QSPI_SS0
  output       flash_dc1_io0        , //QSPI_CLK
  
// UB+power max10  
  input        usb_max_clk          ,
  output[6:0]  usb_cfg              , //GPIOs between ub+power and system,//bit0-> system max10 functional, power max10 internal pull up
  inout        max_scl              , //I2C between UB+power and system max10
  inout        max_sda              ,

//I2C               
  inout        sdm_i2c_sda          , //I2C access to power chips 
  inout        sdm_i2c_scl          ,  
  inout        sysmax_i2c_scl       , //I2C access to Si5338, 5331
  inout        sysmax_i2c_sda       ,
  inout        f2m_i2c_sda          , //max10 -slave ,FM - master, 
  inout        f2m_i2c_scl          ,
  inout        m2f_i2c_sda          , //MAX10 -master, FM -slave 
  inout        m2f_i2c_scl          ,
  inout        i2c_pcie_scl         , //MAX10 PCIE RC connector
  inout        i2c_pcie_sda         ,                                   
 
//system control 
  output       pcie_rc_perstn       , //PCIe root complex reset
  //output       pcie_ep_perstn       , //PCIe endpoint reset
  inout       pcie_ep_perstn       , //PCIe endpoint reset change for fm61 new
  
  input        pcie_rc_prsntn       , //PCIe Add-in card present
  output       pmbus_vid_en         , //FPGA 1.8V - 3.3V I2C buffer enable    
  output       i2c_3_3v_en          , //1.8V - 3.3V I2C buffer enable
  input        si5338_intr          , // GPIO reference clock generator interupt 
  output       si5338_oen           , // GPIO reference clock generator ouput enable
  output       stbyn                , // Temperature sensor standby enable                  
  output       si52202_oen          , // PCIe reference clock generator ouput enable        
  output       si53311_oe           , // XCVR reference clock generator ouput enable        
  output       si53311_clksel       , // XCVR reference clock generator input selection     
  output       clkcleaner_io_tsn    , // IO buffer between clock cleaner and FPGA, controlled by SW[5]
  output       sodimm_io_tsn        , // IO buffer between SODIMM and FPGA         
  output       eth_io_tsn           , // IO buffer between ETH PHY and FPGA   
  output       qsfp_io_tsn          , // IO buffer between QSFP and FPGA      
  output       qsfpdd_io_tsn        , // IO buffer between QSFPDD and FPGA    
  input        ds1339_intn          , // ds1339 intterrup  
  output       max10_overtemp       , //Temperature fault from SYSMAX to power MAX10, high active 
  input        overtempn            , //Temperature sensor over temperature 
  input        temp_alertn          , //Temperature sensor alert 
  input        i2c_enpirion_alertn  , //Enpirion PM bus alert  
  input        i2c_vid_alertn       , //FPGA VCC PM bus alert
  input        i2c_3_3v_alertn      , //FPGA VCCH PM bus alert
  output       fan_rpm              , //PWM to FPGA FAN
  output       opt_fan_rpm            //PWM to optical module FAN, TBD
                                    
);
 
  localparam  proj_id           = 16'h4149;  // ASIC A(gilex)I(SI)
  localparam  max10_version     = 16'h0013;  // 16'h0001 - base version
                                             // 16'h0002 - add extra 20ms delay for hps_dc_resetn after pg_fpga
                                             // 16'h0003 - remove pg_fpga control from flash_dc_rstn
                                             // 16'h0005 - Add filter for all sysmax_pb; use hps_crstn to drive hps_dc_resetn
                                             // 16'h0006 - extend 100ms for PCIE reset
                                             // 16'h0007 - use fpga_gpio[0] to drive pcie_ep_perstn
                                             // 16'h0008 - remove hps_crstn to drive hps_dc_resetn
                                             // 16'h0009 - correct LED behavior
                                             // 16'h000a - correct Fan control behavior
                                             // 16'h000b - use pg_fpga as qsys resetn as bank3 power is same as FPGA
                                             // 16'h000c - avstx32
                                             // 16'h000d - add page sel for avstx32,useless,fpga_sys_resetn stuck at non-avst mode
                                             // 16'h000e - just use avst mode to control PFLII to avoid fpga_sys_resetn stuck in other modes
                                             // 16'h000f - support 4 pgaes
                                             // 16'h0010 - add 100ms delay to release PFLII
                                             // 16'h0011 - use sdm_io15 to drive qspi reset in AS mode
                                             // 16'h0012 - add weak pull-up for sysmax_sw
                                             // 16'h0013 - modify qspi timing constraint
										 
										 
  localparam pg_fpga_delay_cnt       = 16'h1770;   // 6000 x 20.48us = 122.88ms  
  localparam hpc_dc_reset_delay_cnt  = 16'h3E8;    // 1000 x 20.48us = 20.48ms 
  localparam pcie_reset_delay_cnt    = 16'h1388;    // 5000 x 20.48us = 102.4ms 
  
  reg[9:0]  S_clk_div_cnt/* synthesis keep */;
  reg[15:0] S_pg_fpga_delay_cnt/* synthesis keep */;
  reg       S_pg_fpga_delay20ms/* synthesis keep */;									   
  reg       S_pg_fpga_delay100ms/* synthesis keep */;	
  wire      S_clk_div_1024_pulse/* synthesis keep */;								   
  wire      sdm_i2c_sda_in;      
  wire      sdm_i2c_scl_in; 
  wire      sdm_i2c_sda_oe; 
  wire      sdm_i2c_scl_oe; 
  wire      power_max_i2c_sda_in;  //system -> Power max10
  wire      power_max_i2c_scl_in; 
  wire      power_max_i2c_sda_oe; 
  wire      power_max_i2c_scl_oe; 
  wire      system_max_i2c_sda_in; //system -> Si5338,5331
  wire      system_max_i2c_scl_in; 
  wire      system_max_i2c_sda_oe; 
  wire      system_max_i2c_scl_oe; 
  wire      m2f_i2c_sda_in;        //system -> FM
  wire      m2f_i2c_scl_in; 
  wire      m2f_i2c_sda_oe; 
  wire      m2f_i2c_scl_oe; 
  wire      pcie_i2c_sda_in;      //system -> PCIE
  wire      pcie_i2c_scl_in; 
  wire      pcie_i2c_sda_oe; 
  wire      pcie_i2c_scl_oe; 
  wire      factory_load;
  wire[2:0] pwm_duty_ctrl;
  wire      fan_cont_out;
  wire      tsense_alertn;
  wire      system_max_resetn; 
  wire      system_max_resetn2; 
  wire      pfl_flash_access_request;
  wire[2:0] fpga_msel_invert;
  wire      avst_mode/* synthesis keep */;
  wire      as_mode/* synthesis keep */;
  wire      pfl_nreset;
  wire      pgm_config;
  wire      pgm_sel;
  wire[2:0] S_pgm_led;
  wire[2:0] S_fpga_page;
  wire[5:0] S_filtered_sysmax_pb/* synthesis keep */;
  wire      hps_crstn_i;
  wire      S_hps_crstn;  
  wire      S_cpu_resetn_gui;
  wire      S_fpga_reconfign_gui;
  wire[3:0] S_page_sel_gui;
  wire      S_pgm_sel_pb_trig;
  wire      S_max_conf_donen;
  wire      S_max_loadn;
  wire      S_max_errorn;
  wire      en_sdm_bus;
  wire      pcie_rc_perstn_ep; //pcie add
  //wire          pcie_ep_perstn       ; //PCIe endpoint reset change for fm61 new
  assign pgm_sel    = S_filtered_sysmax_pb[4];
  assign pgm_config = S_filtered_sysmax_pb[5];
 
  assign pfl_nreset         = avst_mode ? S_pg_fpga_delay100ms : 1'b0; 
  assign fpga_msel_invert   = {!fpga_msel[2],!fpga_msel[1],!fpga_msel[0]};
  assign avst_mode          = ((fpga_msel_invert == 3'b000) || (fpga_msel_invert == 3'b101) || (fpga_msel_invert == 3'b110)) ? 1'b1 : 1'b0;
  assign as_mode            = ((fpga_msel_invert == 3'b001) || (fpga_msel_invert == 3'b011)) ? 1'b1 : 1'b0;
  assign max10_overtemp     =  !overtempn;
  assign usb_cfg[0]         = 1'b0;  //system max10 functional
  assign system_max_resetn  = pg_sysmax & S_filtered_sysmax_pb[0]; 
  assign system_max_resetn2 = pg_fpga & S_filtered_sysmax_pb[0]; 
  assign factory_load       = sysmax_sw[0]; 
//assign clksel_156m        = sysmax_sw[1]; can'f find clksel_156m or si53311_clksel
//assign                    = sysmax_sw[2]; //reserved
//assign                    = sysmax_sw[3]; //reserved
  assign i2c_3_3v_en        = sysmax_sw[4];    
  assign clkcleaner_io_tsn  = sysmax_sw[5]; 

  assign fpga_sys_resetn    = pg_fpga ? (S_fpga_resetn && S_filtered_sysmax_pb[1]) : 1'bz;
 
  assign pcie_rc_perstn_ep     = S_pg_fpga_delay100ms & fpga_gpio[0] ; //  fpga_gpio[0]: low active, 0: reset
  assign pcie_rc_perstn     = en_sdm_bus ? pcie_ep_perstn : pcie_rc_perstn_ep ; //  change for fm61 new
  assign pcie_ep_perstn     = en_sdm_bus ? 1'bz : pcie_rc_perstn_ep ;
//  
  assign stbyn              = 1'bZ;
  assign si52202_oen        = !pg_fpga;
  assign si5338_oen         = !pg_fpga;
  assign pmbus_vid_en       = pg_fpga;
  assign si53311_oe         = pg_fpga;
  assign si53311_clksel     = sysmax_sw[1];  
  assign sodimm_io_tsn      = pg_fpga;
  assign eth_io_tsn         = pg_fpga;
  assign qsfp_io_tsn        = pg_fpga;
  assign qsfpdd_io_tsn      = pg_fpga;  
   
  
 //do not trigger flash reset for now
 //for S10 soc es silicon, known issue for gap between flash_reset and nconfig 
 // -> flash reset needs be released earlier before FPGA SDM, otherwise AS will fail
 // -> use sdm_io15 to drive flash reset for FM RevB and later, https://hsdes.intel.com/appstore/article/#/16011817805
   
 //Add 20ms delay to release hps_dc_resetn after pg_fpga
  assign flash_dc1_rstn     = 1'bZ;//pg_fpga & S_filtered_sysmax_pb[2]; // 
  assign flash_dc0_rstn     = as_mode ? (sdm_io15 == 1'b0 ? 1'b0: 1'bZ):1'bZ;  //don't drive to 1 to avoid conflict with future hardware change
  assign hps_dc_resetn      = S_pg_fpga_delay20ms;// && hps_crstn_i  ;//&& S_filtered_sysmax_pb[2]; //; just keep power up reset
  assign S_hps_crstn        = pg_fpga ? (S_filtered_sysmax_pb[2] & hps_proc_rstn): 1'b0 ; //hps cold reset
  assign hps_crstn          = S_hps_crstn ? 1'bZ : 1'b0;
  assign hps_crstn_i        = hps_crstn;
  assign sdm_i2c_scl_in     = sdm_i2c_scl; 
  assign sdm_i2c_scl        = sdm_i2c_scl_oe ? 1'b0 : 1'bZ;		
  assign sdm_i2c_sda_in     = sdm_i2c_sda; 
  assign sdm_i2c_sda        = sdm_i2c_sda_oe ? 1'b0 : 1'bZ;	 
  
  assign system_max_i2c_scl_in = sysmax_i2c_scl; 
  assign sysmax_i2c_scl        = system_max_i2c_scl_oe ? 1'b0 : 1'bZ;		
  assign system_max_i2c_sda_in = sysmax_i2c_sda; 
  assign sysmax_i2c_sda        = system_max_i2c_sda_oe ? 1'b0 : 1'bZ;	 
  
  assign max_i2c_scl_in        = max_scl; 
  assign max_scl               = max_i2c_scl_oe ? 1'b0 : 1'bZ;		
  assign max_i2c_sda_in        = max_sda; 
  assign max_sda               = max_i2c_sda_oe ? 1'b0 : 1'bZ;	 
  
  assign m2f_i2c_scl_in        = m2f_i2c_scl; 
  assign m2f_i2c_scl           = m2f_i2c_scl_oe ? 1'b0 : 1'bZ;		
  assign m2f_i2c_sda_in        = m2f_i2c_sda; 
  assign m2f_i2c_sda           = m2f_i2c_sda_oe ? 1'b0 : 1'bZ;	 
  
  assign pcie_i2c_scl_in       = i2c_pcie_scl; 
  assign i2c_pcie_scl          = pcie_i2c_scl_oe ? 1'b0 : 1'bZ;		
  assign pcie_i2c_sda_in       = i2c_pcie_sda; 
  assign i2c_pcie_sda          = pcie_i2c_sda_oe ? 1'b0 : 1'bZ;	 
    
  //LED 0:ON, 1:OFF
  assign sysmax_led_g[2:0] = S_pgm_led ;        // page indication
  assign sysmax_led_g[3]   = S_max_loadn ;      // ON while loading FPP design from flash
  assign sysmax_led_g[4]   = S_max_conf_donen ; // ON after PFL Configuration PASS
  assign sysmax_led_y[0]   = avst_mode ? S_max_errorn : 1'b1; //On if avst configuration failure
  assign sysmax_led_y[1]   = overtempn ; 
  
  //add extra delay for hps_dc_resetn
  assign S_clk_div_1024_pulse = (S_clk_div_cnt == 10'd1023) ? 1'b1: 1'b0; 
  
  always @(posedge clk_50m_sysmax or negedge system_max_resetn)
  begin
    if(!system_max_resetn)
      S_clk_div_cnt <= 10'd0;
    else
      S_clk_div_cnt <= S_clk_div_cnt +1'b1;
  end
  
  always @(posedge clk_50m_sysmax or negedge system_max_resetn)
  begin
    if(!system_max_resetn) 
	  S_pg_fpga_delay20ms <= 1'b0;
    else if(!pg_fpga)
	  S_pg_fpga_delay20ms <= 1'b0;
	else if(S_pg_fpga_delay_cnt == hpc_dc_reset_delay_cnt)
	  S_pg_fpga_delay20ms <= 1'b1;
	else
	  S_pg_fpga_delay20ms <= S_pg_fpga_delay20ms;
  end
  
  always @(posedge clk_50m_sysmax or negedge system_max_resetn)
  begin
    if(!system_max_resetn) 
	  S_pg_fpga_delay100ms <= 1'b0;
    else if(!pg_fpga)
	  S_pg_fpga_delay100ms <= 1'b0;
	else if(S_pg_fpga_delay_cnt == pcie_reset_delay_cnt)
	  S_pg_fpga_delay100ms <= 1'b1;
	else
	  S_pg_fpga_delay100ms <= S_pg_fpga_delay100ms;
  end
  
  always @(posedge clk_50m_sysmax or negedge system_max_resetn)
  begin
    if(!system_max_resetn)
      S_pg_fpga_delay_cnt <= 16'd0;
    else if(!pg_fpga)
      S_pg_fpga_delay_cnt <= 16'd0;
    else if(S_clk_div_1024_pulse)
      begin
        if(S_pg_fpga_delay_cnt == pg_fpga_delay_cnt)
          S_pg_fpga_delay_cnt <= S_pg_fpga_delay_cnt;
        else
          S_pg_fpga_delay_cnt <= S_pg_fpga_delay_cnt +1'b1;
      end
    else
      S_pg_fpga_delay_cnt <= S_pg_fpga_delay_cnt;
  end
  
  //Filter push buttons  
  pb_filter #(
    .FILTER_WIDTH       (6'd8), 
    .INIT_BIT           (1'b1))
  pb_filter_inst(
    .I_reset_n          (pg_sysmax            ),  //don't use system_max_resetn, otherwise sysmax_pb[0] will be affected 
    .I_clk              (clk_50m_sysmax  	  ),
    .I_user_pb          (sysmax_pb            ),
    .O_filtered_user_pb (S_filtered_sysmax_pb ) 
  );  
  
  //-----------------------------------------------------------------------------------------------------------------
  // Fan control interface: PWM Frequency 25KHz
  //-----------------------------------------------------------------------------------------------------------------
  fan_controller fan_ctrl_inst(
    .i_clk           (clk_50m_sysmax       ),  
    .i_reset_n       (system_max_resetn    ),  
    .i_pwm_duty_ctrl (pwm_duty_ctrl        ), // Duty cycle control, 3'b100 (60%) by default
    .o_fan_cont_out  (fan_cont_out         )  // Fan control output, low active
    );
  
  //Fan control 
  assign fan_rpm = ((overtempn && fan_cont_out) == 1'b0) ? 1'b1 : 1'b0; // 1: Fan ON; 0: Fan OFF
  
  max_pfl_ctrl max_pfl_ctrl_inst( 
    .i_reset_n           (system_max_resetn2   ),
    .i_pfl_nreset        (pfl_nreset           ),
    .i_system_clk        (clk_50m_sysmax       ),
    .i_clk_div_1024_pulse(S_clk_div_1024_pulse ),
    .i_cpu_resetn_gui    (S_cpu_resetn_gui     ),
    .i_fpga_reconfign_gui(S_fpga_reconfign_gui ),
    .i_page_sel_gui      (S_page_sel_gui       ),
    .i_factory_load      (factory_load         ),
    .i_pgm_sel           (pgm_sel              ),
    .i_pgm_config        (pgm_config           ),
    .i_fpga_conf_done    (sdm_io16             ),
    .i_fpga_nstatus      (fpga_nstatus         ),
    .flash_io0           (flash_dc1_io1        ),
    .flash_io1           (flash_dc1_io3        ),
    .flash_io2           (flash_dc1_io4        ),
    .flash_io3           (flash_dc1_io5        ),
    .flash_ncs           (flash_dc1_io2        ),
    .flash_sck           (flash_dc1_io0        ),
    .o_avst_valid        (avst_data_valid      ),
    .i_avst_ready        (avst_ready           ),
    .o_avst_data         (avst_d               ),
    .o_avst_clk          (avst_clk             ),
    .o_fpga_nconfig      (fpga_nconfig         ),
    .o_max_conf_donen    (S_max_conf_donen     ),
    .o_max_loadn         (S_max_loadn          ),
    .O_max_errorn        (S_max_errorn         ),
    .o_fpga_pgm          (S_fpga_page          ),
    .o_pgm_led           (S_pgm_led            ),
    .o_pgm_sel_pb_trig	 (S_pgm_sel_pb_trig    ),
    .o_cpu_resetn        (S_fpga_resetn        )
    );                                                             
   
   
  //pg_sysmax doesn't mean the whole max10 power banks are ok, the I2C interfaces in bank3 use FPGA 1.8V power
  //use system_max_resetn2 to avoid I2c test failure.
  q_sys q_sys_inst(
  	.clk_clk                              (clk_50m_sysmax          ),  
  	.reset_reset_n                        (system_max_resetn2      ), 
  	.i2c_0_i2c_serial_sda_in              (sdm_i2c_sda_in          ),
  	.i2c_0_i2c_serial_scl_in              (sdm_i2c_scl_in          ),
  	.i2c_0_i2c_serial_sda_oe              (sdm_i2c_sda_oe          ),
  	.i2c_0_i2c_serial_scl_oe              (sdm_i2c_scl_oe          ),
  	.i2c_1_i2c_serial_sda_in              (system_max_i2c_sda_in   ),
  	.i2c_1_i2c_serial_scl_in              (system_max_i2c_scl_in   ),
  	.i2c_1_i2c_serial_sda_oe              (system_max_i2c_sda_oe   ),
  	.i2c_1_i2c_serial_scl_oe              (system_max_i2c_scl_oe   ),
  	.i2c_2_i2c_serial_sda_in              (m2f_i2c_sda_in          ),
  	.i2c_2_i2c_serial_scl_in              (m2f_i2c_scl_in          ),
  	.i2c_2_i2c_serial_sda_oe              (m2f_i2c_sda_oe          ),
  	.i2c_2_i2c_serial_scl_oe              (m2f_i2c_scl_oe          ),
  	.i2c_3_i2c_serial_sda_in              (pcie_i2c_sda_in         ),
  	.i2c_3_i2c_serial_scl_in              (pcie_i2c_scl_in         ),
  	.i2c_3_i2c_serial_sda_oe              (pcie_i2c_sda_oe         ),
  	.i2c_3_i2c_serial_scl_oe              (pcie_i2c_scl_oe         ),
  	.i2c_power_max10_i2c_serial_sda_in    (max_i2c_sda_in          ),
  	.i2c_power_max10_i2c_serial_scl_in    (max_i2c_scl_in          ),
  	.i2c_power_max10_i2c_serial_sda_oe    (max_i2c_sda_oe          ),
  	.i2c_power_max10_i2c_serial_scl_oe    (max_i2c_scl_oe          ),
  	.pio_dipsw_external_connection_export (sysmax_sw               ),
  	.pio_led_g_external_connection_export (sysmax_led_g            ),              
  	.pio_led_y_external_connection_export (sysmax_led_y            ),              
  	.pio_pb_external_connection_export    (S_filtered_sysmax_pb    ),
  	.system_max_id_0_control_gui_export   (                        ),  
  	.system_max_id_0_temp_data_export     (                        ),
  	.system_max_id_0_cpu_resetn_export    (S_cpu_resetn_gui        ),
  	.system_max_id_0_fpga_pgm_export      (S_fpga_page             ),
  	.system_max_id_0_fpga_reconfign_export(S_fpga_reconfign_gui    ),
  	.system_max_id_0_page_sel_export      (S_page_sel_gui          ),
  	.system_max_id_0_page_sel_pb_export   (S_pgm_sel_pb_trig       ), 
  	.system_max_id_0_pwm_duty_ctrl_export (pwm_duty_ctrl           ),
  	.system_max_id_0_project_id_export    (proj_id                 ),
  	.system_max_id_0_version_export       (max10_version           ),
	.pio_en_sdm_external_connection_export(en_sdm_bus)
  	);
  
endmodule
